/*
    libMakeMKV - MKV multiplexer library

    Copyright (C) 2009 GuinpinSoft inc <libmkv@makemkv.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*/
#include <lgpl/zlibabi.h>
#include <zlib.h>
#include <string.h>

extern voidpf ZLIB_alloc_func (voidpf opaque, uInt items, uInt size);
extern void ZLIB_free_func  (voidpf opaque, voidpf address);

static int ZLIB_translateError(int err)
{
    switch(err)
    {
    case Z_OK: return ZLIB_Z_OK;
    case Z_STREAM_END: return ZLIB_Z_STREAM_END;
    case Z_NEED_DICT: return ZLIB_Z_NEED_DICT;
    default:
        break;
    }
    if (err>0)
    {
        return -100;
    }
    return err;
}

int ZLIB_compressBest(uint8_t* dest,unsigned int* destLen,const uint8_t* source,unsigned int sourceLen)
{
    int err;
    uLongf dlen;
    
    dlen = *destLen;
    err = compress2(dest,&dlen,source,sourceLen,Z_BEST_COMPRESSION);
    *destLen = (unsigned int)dlen;
    
    return ZLIB_translateError(err);
}

unsigned int ZLIB_compressBound(unsigned int sourceLen)
{
    return compressBound(sourceLen);
}

int ZLIB_uncompress(uint8_t* dest,unsigned int* destLen,const uint8_t* source,unsigned int sourceLen)
{
    int err;
    uLongf dlen;

    dlen = *destLen;
    err = uncompress(dest,&dlen,source,sourceLen);
    *destLen = (unsigned int)dlen;
    
    return ZLIB_translateError(err);
}

unsigned int ZLIB_sizeof_Z_STREAM()
{
    return (((sizeof(z_stream) + (sizeof(void*)-1))/sizeof(void*))*sizeof(void*));
}

void ZLIB_get_stream(ZLIB_Z_STREAM* zstream,int in,uint8_t** next,uint32_t* avail,uint64_t* total)
{
    z_stream* stream = (z_stream*)zstream;
    if (in)
    {
        *next  = (uint8_t*) stream->next_in;
        *avail = (uint32_t) stream->avail_in;
        *total = (uint64_t) stream->total_in;
    } else {
        *next  = (uint8_t*) stream->next_out;
        *avail = (uint32_t) stream->avail_out;
        *total = (uint64_t) stream->total_out;
    }
}

void ZLIB_set_stream(ZLIB_Z_STREAM* zstream,int in,uint8_t* next,uint32_t avail)
{
    z_stream* stream = (z_stream*)zstream;
    if (in)
    {
        stream->next_in = (Bytef*) next;
        stream->avail_in= (uInt) avail;
    } else {
        stream->next_out = (Bytef*) next;
        stream->avail_out= (uInt) avail;
    }
}

int ZLIB_deflateInitBest(ZLIB_Z_STREAM* zstream,int windowBits)
{
    z_stream* stream = (z_stream*)zstream;

    memset(stream,0,sizeof(z_stream));
    stream->zalloc = ZLIB_alloc_func;
    stream->zfree = ZLIB_free_func;

    return ZLIB_translateError(deflateInit2( stream , Z_BEST_COMPRESSION , Z_DEFLATED , windowBits , 9 , Z_DEFAULT_STRATEGY ));
}

int ZLIB_deflateAsync(ZLIB_Z_STREAM* zstream,int finish)
{
    return ZLIB_translateError( deflate( ((z_stream*)zstream) , ((finish!=0) ? Z_FINISH : Z_NO_FLUSH) ) );
}

int ZLIB_deflateEnd(ZLIB_Z_STREAM* zstream)
{
    return ZLIB_translateError( deflateEnd( ((z_stream*)zstream) ) );
}

